/****************************************************************************

**  Gcssamer - Lightweight CSS Inspector for Firefox

**  Copyright (c) 2008  Charles Melhorn

**

**  This file is part of Gcssamer.

**

**  Gcssamer is free software; you can redistribute it and/or modify

**  it under the terms of the GNU General Public License as published by

**  the Free Software Foundation; either version 2 of the License, or

**  (at your option) any later version.

**

**  Gcssamer is distributed in the hope that it will be useful,

**  but WITHOUT ANY WARRANTY; without even the implied warranty of

**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

**  GNU General Public License for more details.

**

**  You should have received a copy of the GNU General Public License

**  along with this program; if not, write to the Free Software

**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

**

**  Or see the GNU web site at: http://www.gnu.org

**

****************************************************************************/ 



// extend Array object
if (!Array.hasValue)
  Array.prototype.hasValue = function(aValue)
  {
    for(var idx=0; idx<this.length; idx++)
     {
       if (this[idx] == aValue)
         return true
     }
    return false;
  };
  

// globals, constants, services
var   gCurrentElement = null;
const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                         .getService(Components.interfaces.nsIClipboardHelper);
const COL_SELECTEDFLAG = 0, COL_PROPSTRING = 1, COL_PROPVALUE = 2; // indices for style_data array



function truncateString(aString, aLength)
{
  return aString.substr(0, aLength) + (aString.length > aLength ? "..." : "");
}


function initCSSInfo(aElement)
{
  cssInfoTreeView.clear();

  document.getElementById('target-tag-type').value = aElement.tagName;
  document.getElementById('target-innerhtml').value = truncateString(aElement.innerHTML, 50);

  var style_string = "";
  var prop_string = "";

  // get ComputedStyle object for our element - in unfocused state
  aElement.blur();
  cs = aElement.ownerDocument.defaultView.getComputedStyle(aElement, null);

  // get preferred css properties
  var favorite_props = gGCSSAMERUserPrefUtils.getPref('cssprops');

  if (favorite_props == "")
    favorite_props = [];
  else
    favorite_props = favorite_props.split(';');

  var propset = document.getElementById('propset-rgroup').selectedItem.value;

  if (propset == "selected")
   {
     for (var propidx=0; propidx < favorite_props.length; propidx++)
      {
        prop_string = favorite_props[propidx];
        cssInfoTreeView.style_data[propidx]= new Array(3);
        cssInfoTreeView.style_data[propidx][COL_SELECTEDFLAG] = true;
        cssInfoTreeView.style_data[propidx][COL_PROPSTRING] = prop_string;
        cssInfoTreeView.style_data[propidx][COL_PROPVALUE] = cs.getPropertyValue(prop_string);
      }
   }
  else
   {
     for (var propidx=0; propidx < cs.length; propidx++)
      {
        prop_string = cs.item(propidx);
        cssInfoTreeView.style_data[propidx]= new Array(3);
        cssInfoTreeView.style_data[propidx][COL_SELECTEDFLAG] = favorite_props.hasValue(prop_string);
        cssInfoTreeView.style_data[propidx][COL_PROPSTRING] = prop_string;
        cssInfoTreeView.style_data[propidx][COL_PROPVALUE] = cs.getPropertyValue(prop_string);
      }
   }

  // force tree to redraw
  cssInfoTreeView.tree.rowCountChanged(0, cssInfoTreeView.style_data.length);
  cssInfoTreeView.tree.invalidate();

  gCurrentElement = aElement;

  if (gCurrentElement.parentNode == null || gCurrentElement.parentNode.nodeType != Node.ELEMENT_NODE)
    document.getElementById('up-to-parent-btn').setAttribute('disabled', "true");
}


function initStyleSheetInfo(aElement)
{
  var stylesheet_lbox = document.getElementById('stylesheet-lbox');
  var ssList = aElement.ownerDocument.styleSheets;
  var lc = null, li = null;

  for(var ss=0; ss < ssList.length; ss++)
   {
      li = document.createElement('listitem');

      lc = document.createElement('listcell');
      lc.setAttribute('label', ssList.item(ss).href);
      li.appendChild(lc);

      lc = document.createElement('listcell');
      lc.setAttribute('label', ssList.item(ss).media.item(0));
      li.appendChild(lc);

      stylesheet_lbox.appendChild(li);
   }
}


function openStyleSheet()
{
  var ss_url = document.getElementById('stylesheet-lbox').selectedItem.firstChild.getAttribute('label');
  window.opener.open(ss_url);
}


function copyStyleSheetLocation()
{
  var ss_url = document.getElementById('stylesheet-lbox').selectedItem.firstChild.getAttribute('label');
  gClipboardHelper.copyString(ss_url);
}


function upToParentElement()
{
  // we protect against null parent nodes by disabling the up-to-parent button in initCSSInfo()
  initCSSInfo(gCurrentElement.parentNode);
}


// derived from Firefox 3.0: pageInfo.js
// define a js object to implement nsITreeView

cssInfoTreeView = {
  style_data: [],

  set rowCount(c) { throw "rowCount is a readonly property"; },
  get rowCount() { return this.style_data.length },

  clear: function()
  {
    if (this.tree)
      this.tree.rowCountChanged(0, -this.style_data.length);
    this.style_data = [];
  },

  setTree: function(tree)
  {
    this.tree = tree;
  },

  getCellText: function(row, column)
  {
    return (column.id != "propselected" ? this.style_data[row][COL_PROPSTRING] + ": " +
                                          this.style_data[row][COL_PROPVALUE] : "");
  },

  setCellValue: function(row, column, value){},

  setCellText: function(row, column, value){},

  rowCountChanged: function(index, count){},

  invalidate: function()
  {
    this.tree.invalidate();
  },

  handleCopy: function(row)
  {
    return (this.style_data[row][COL_PROPSTRING] + ": " + this.style_data[row][COL_PROPVALUE] + ";");
  },

  performActionOnRow: function(action, row)
  {
    if (action == "copy") {
      var data = this.handleCopy(row)

      // per nsITreeView API, return type for this method is void, so make data available via tree attribute
      this.tree.treeBody.parentNode.setAttribute("copybuffer", data);
    }
  },

  getRowProperties: function(row, prop) { },

  getCellProperties: function(row,col,props)
  {
    if (this.style_data[row][COL_SELECTEDFLAG]){
      var aserv=Components.classes["@mozilla.org/atom-service;1"].
              getService(Components.interfaces.nsIAtomService);
      props.AppendElement(aserv.getAtom("on"));
    }
  },

  cycleCell: function(row, column)
  {
    this.style_data[row][COL_SELECTEDFLAG] = !this.style_data[row][COL_SELECTEDFLAG];

    var favorite_props = [];
    for (var idx=0; idx < this.style_data.length; idx++)
     {
       if (this.style_data[idx][COL_SELECTEDFLAG] == true)
         favorite_props.push(this.style_data[idx][COL_PROPSTRING])
     }
    gGCSSAMERUserPrefUtils.setPref('cssprops', favorite_props.join(';'));
  },

  getColumnProperties: function(column, prop) { },
  isContainer: function(index) { return false; },
  isContainerOpen: function(index) { return false; },
  isSeparator: function(index) { return false; },
  isSorted: function() { },
  canDrop: function(index, orientation) { return false; },
  drop: function(row, orientation) { return false; },
  getParentIndex: function(index) { return 0; },
  hasNextSibling: function(index, after) { return false; },
  getLevel: function(index) { return 0; },
  getImageSrc: function(row, column) { },
  getProgressMode: function(row, column) { },
  getCellValue: function(row, column) { },
  toggleOpenState: function(index) { },
  cycleHeader: function(col) { },
  selectionChanged: function() { },
  isEditable: function(row, column) { return false; },
  isSelectable: function(row, column) { return false; },
  performAction: function(action) { },
  performActionOnCell: function(action, row, column) { }
};


// from Firefox 3.0: pageInfo.js
// generic tree row copy handler

function doCopy()
{
  if (!gClipboardHelper)
    return;

  var elem = document.commandDispatcher.focusedElement;

  if (elem && "treeBoxObject" in elem) {
    var view = elem.view;
    var selection = view.selection;
    var text = [], tmp = '';
    var min = {}, max = {};

    var count = selection.getRangeCount();

    for (var i = 0; i < count; i++) {
      selection.getRangeAt(i, min, max);

      for (var row = min.value; row <= max.value; row++) {
        view.performActionOnRow("copy", row);

        tmp = elem.getAttribute("copybuffer");
        if (tmp)
          text.push(tmp);
        elem.removeAttribute("copybuffer");
      }
    }
    gClipboardHelper.copyString(text.join("\n"));
  }
}

